{+doc
+NOMBRE:   uParqueEolico
+CREACION: 22/12/2006
+AUTORES:  rch
+REGISTRO:
+TIPO: Unidad Pascal.
+PROPOSITO:   Definición del actor TParqueEolico
+PROYECTO: SimSEEU

+REVISION:
+AUTOR:
+DESCRIPCION:

-doc}

unit uParqueEolico_vxy;


interface

uses
  Classes,
  SysUtils,
  uFechas,
  uGeneradores,
  uFuentesAleatorias,
  uGlobs,
  uNodos,
  uCosa,
  usimplex,
  uFichasLPD,
  ufuncionesreales,
  xMatDefs;

resourcestring
  mesFuenteAleatConPaso1h =
    'TParqueEolico, necesita una fuente aleatoria con paso de sorteo = 1h. ';
  mesElParque = 'El parque ';
  mesConectadoAFuente = ' está conectado a la fuente ';
  mesConPasoDeSorteo = ' con paso de sorteo = ';
  rsParqueEolico_vxy = 'Parque eólico_vxy';

type

  { TParqueEolico_vxy }

  TParqueEolico_vxy = class(TGeneradorPostizador)
  public
    PagoPorEnergiaDisponible_USD_MWh: NReal;
    PagoPorEnergiaEntregada_USD_MWh: NReal; // [USD/MWh] para cálculo de CAD

    // fuente de viento
    fuenteDeViento: TFuenteAleatoria;
    nombreBorne_vx, nombreBorne_vy: string;

    kBorneVelocidad_vx, kBorneVelocidad_vy: integer;

    {$IFDEF CALC_DEMANDA_NETA}
    kBornePotencia: integer;
    {$ELSE}
    kBornePPA: TDAofNInt; // Potencia del Poste Actual
    {$ENDIF}

    // parámetros editables
    fdisp: NReal;
    tRepHoras: NReal;
    fxPerdidasAerodinamicas: TDAofNReal; // para guadar fPerdiasAerodinamicas * MultV
    fPerdidasAerodinamicas: TDAofNReal;  // por dirección.
    fMultV: NReal;

    CurvaVP: TArrayOfFVectR;


    VVel_x, VVel_y: NReal;


    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean; xFDV: TFuenteAleatoria;
  xNombreBorne_vx, xNombreBorne_vy: string; xfdisp: NReal; tRepHoras: NReal;
  xfPerdidasAerodinamicas: TDAOfNReal; xfMultV: NReal;
  xCurvaVelocidadPotencia_Pot: TArrayOfFVectR;
  xPagoPorEnergiaDisponible_USD_MWh, xPagoPorEnergiaEntregada_USD_MWh: NReal;
  xflg_RestarParaPostizado: boolean; TonCO2xMWh: NReal; LowCostMustRun,
  CleanDevelopmentMechanism: boolean);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PrepararMemoria(globs: TGlobs); override;

    function PotenciaFirme: NReal; override;

    function InfoAd_: string; override;

    class function DescClase: string; override;

    function get_pa_FD( kTipoUnidad: integer ): NReal; override;
    function get_pa_TMR( kTipoUnidad: integer ): NReal; override;
    procedure SorteosDelPaso(sortear: boolean); override;
    procedure PrepararPaso_ps; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;

    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;
    {$IFDEF SPXCONLOG}
    procedure spx_NombrarVariables(s: TSimplex); override;
    {$ENDIF}
    procedure Free; override;
    procedure PubliVars; override;
    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    function getNombreVar(ivar: integer; var nombre: string): boolean; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
  end;

procedure AlInicio;
procedure AlFinal;

implementation


constructor TParqueEolico_vxy.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean;
  xFDV: TFuenteAleatoria;
  xNombreBorne_vx, xNombreBorne_vy: string; xfdisp: NReal; tRepHoras: NReal;
  xfPerdidasAerodinamicas: TDAOfNReal; xfMultV: NReal;
  xCurvaVelocidadPotencia_Pot: TArrayOfFVectR;
  xPagoPorEnergiaDisponible_USD_MWh, xPagoPorEnergiaEntregada_USD_MWh: NReal;
  xflg_RestarParaPostizado: boolean;
  TonCO2xMWh: NReal; LowCostMustRun, CleanDevelopmentMechanism: boolean);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, nodo,
    TonCO2xMWh, LowCostMustRun, CleanDevelopmentMechanism, xflg_RestarParaPostizado,   flg_CalcularGradienteDeInversion );

  fdisp := xfdisp;
  self.tRepHoras := tRepHoras;
  fPerdidasAerodinamicas := xfPerdidasAerodinamicas;
  fMultV := xfMultV;
  CurvaVP := xCurvaVelocidadPotencia_Pot;
  FuenteDeViento := xFDV;
  NombreBorne_vx := xNombreBorne_vx;
  NombreBorne_vy := xNombreBorne_vy;
  kBorneVelocidad_vx := -1;
  kBorneVelocidad_vy := -1;
  PagoPorEnergiaDisponible_USD_MWh:= xPagoPorEnergiaDisponible_USD_MWh;
  PagoPorEnergiaEntregada_USD_MWh := xPagoPorEnergiaEntregada_USD_MWh;
end;

procedure TParqueEolico_vxy.Free;
var
  k: integer;
begin
  for k := 0 to 15 do
    CurvaVP[k].Free;
  {$IFNDEF CALC_DEMANDA_NETA}
  SetLength(kBornePPA, 0);
  {$ENDIF}
  SetLength(fxPerdidasAerodinamicas, 0);
  SetLength(fPerdidasAerodinamicas, 0);
  SetLength(PPA, 0);
  inherited Free;
end;

procedure TParqueEolico_vxy.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('PotenciaGenerable', '[MW]', 6, 1, PPA, True, False);
  PublicarVariableNR('VVel_x', '[m/s]', 6, 2, VVel_x, True);
  PublicarVariableNR('VVel_y', '[m/s]', 6, 2, VVel_y, True);
  PublicarVariableNI('NMaqsDisponibles', '-', NMaquinasDisponibles, True);
  PublicarVariableNR('CostoDirectoDelPaso', '[USD]', 6, 2, costoDirectoDelPaso, True);
end;

constructor TParqueEolico_vxy.Create_ReadFromText(f: TArchiTexto);
var
  kdir: integer;
  tvr: TFVectR;
begin
  inherited Create_ReadFromText(f);

  PagoPorEnergiaDisponible_USD_MWh:= 0;
  PagoPorEnergiaEntregada_USD_MWh:= 0;

  f.IniciarLecturaRetrasada;
  f.rd('fdisp', fdisp);
  f.rd('tRepHoras', tRepHoras);
  f.rd('fPerdidasAerodinamicas', fPerdidasAerodinamicas);
  if f.Version < 39 then
    fMultV := 1.0
  else
    f.rd('fMultV', fMultV);

  setlength(CurvaVP, 16);
  if f.Version < 40 then
    f.rd('CurvaVP', TCosa(tvr))
  else
  begin
    if f.Version = 40 then
    begin // corrigo error de tipeo
      f.rd('CurvaVP_N', TCosa(CurvaVP[0]));
      f.rd('CurvaVP_NNE', TCosa(CurvaVP[1]));
      f.rd('CurvaVP_NE', TCosa(CurvaVP[2]));
      f.rd('CurvaVP_ENE', TCosa(CurvaVP[3]));
      f.rd('CurvaVP_ESE', TCosa(CurvaVP[4]));
      f.rd('CurvaVP_SE', TCosa(CurvaVP[5]));
      f.rd('CurvaVP_SSE', TCosa(CurvaVP[6]));
      f.rd('CurvaVP_S', TCosa(CurvaVP[7]));
      f.rd('CurvaVP_SSO', TCosa(CurvaVP[8]));
      f.rd('CurvaVP_SO', TCosa(CurvaVP[9]));
      f.rd('CurvaVP_OSO', TCosa(CurvaVP[10]));
      f.rd('CurvaVP_O', TCosa(CurvaVP[11]));
      f.rd('CurvaVP_ONO', TCosa(CurvaVP[12]));
      f.rd('CurvaVP_NO', TCosa(CurvaVP[13]));
      f.rd('CurvaVP_NNO', TCosa(CurvaVP[14]));
      f.rd('CurvaVP_N', TCosa(CurvaVP[15]));
    end
    else
    begin
      f.rd('CurvaVP_N', TCosa(CurvaVP[0]));
      f.rd('CurvaVP_NNE', TCosa(CurvaVP[1]));
      f.rd('CurvaVP_NE', TCosa(CurvaVP[2]));
      f.rd('CurvaVP_ENE', TCosa(CurvaVP[3]));
      f.rd('CurvaVP_E', TCosa(CurvaVP[4]));
      f.rd('CurvaVP_ESE', TCosa(CurvaVP[5]));
      f.rd('CurvaVP_SE', TCosa(CurvaVP[6]));
      f.rd('CurvaVP_SSE', TCosa(CurvaVP[7]));
      f.rd('CurvaVP_S', TCosa(CurvaVP[8]));
      f.rd('CurvaVP_SSO', TCosa(CurvaVP[9]));
      f.rd('CurvaVP_SO', TCosa(CurvaVP[10]));
      f.rd('CurvaVP_OSO', TCosa(CurvaVP[11]));
      f.rd('CurvaVP_O', TCosa(CurvaVP[12]));
      f.rd('CurvaVP_ONO', TCosa(CurvaVP[13]));
      f.rd('CurvaVP_NO', TCosa(CurvaVP[14]));
      f.rd('CurvaVP_NNO', TCosa(CurvaVP[15]));
    end;
  end;

  f.rdReferencia('fuenteDeViento', TCosa(fuenteDeViento), Self);
  f.rd('nombreBorne_vx', nombreBorne_vx);
  f.rd('nombreBorne_vy', nombreBorne_vy);

  if f.version < 98 then
    f.rd('pagoPorEnergia', PagoPorEnergiaEntregada_USD_MWh)
  else if f.version < 103 then
  begin
    f.rd('PagoPorDisponibilidad_USD_MWh', PagoPorEnergiaDisponible_USD_MWh);
    f.rd('PagoPorEnergia_USD_MWh', PagoPorEnergiaEntregada_USD_MWh)
  end
  else
  begin
    f.rd('PagoPorEnergiaDisponible_USD_MWh', PagoPorEnergiaDisponible_USD_MWh );
    f.rd('PagoPorEnergiaEntregada_USD_MWh', PagoPorEnergiaEntregada_USD_MWh)
  end;


  f.EjecutarLectura;

  if f.Version < 40 then
  begin
    for kdir := 0 to 15 do
      CurvaVP[kdir] := tvr.Create_Clone as TFVectR;
    tvr.Free;
  end;

  kBorneVelocidad_vx := -1;
  kBorneVelocidad_vy := -1;
end;

procedure TParqueEolico_vxy.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('fdisp', fdisp);
  f.wr('tRepHoras', tRepHoras);
  f.wr('fPerdidasAerodinamicas', fPerdidasAerodinamicas);
  f.wr('fMultV', fMultV);

  f.wr('CurvaVP_N', TCosa(CurvaVP[0]));
  f.wr('CurvaVP_NNE', TCosa(CurvaVP[1]));
  f.wr('CurvaVP_NE', TCosa(CurvaVP[2]));
  f.wr('CurvaVP_ENE', TCosa(CurvaVP[3]));
  f.wr('CurvaVP_E', TCosa(CurvaVP[4]));
  f.wr('CurvaVP_ESE', TCosa(CurvaVP[5]));
  f.wr('CurvaVP_SE', TCosa(CurvaVP[6]));
  f.wr('CurvaVP_SSE', TCosa(CurvaVP[7]));
  f.wr('CurvaVP_S', TCosa(CurvaVP[8]));
  f.wr('CurvaVP_SSO', TCosa(CurvaVP[9]));
  f.wr('CurvaVP_SO', TCosa(CurvaVP[10]));
  f.wr('CurvaVP_OSO', TCosa(CurvaVP[11]));
  f.wr('CurvaVP_O', TCosa(CurvaVP[12]));
  f.wr('CurvaVP_ONO', TCosa(CurvaVP[13]));
  f.wr('CurvaVP_NO', TCosa(CurvaVP[14]));
  f.wr('CurvaVP_NNO', TCosa(CurvaVP[15]));

  f.wrReferencia('fuenteDeViento', fuenteDeViento);
  f.wr('nombreBorne_vx', nombreBorne_vx);
  f.wr('nombreBorne_vy', nombreBorne_vy);
  f.wr('PagoPorEnergiaDisponible_USD_MWh', PagoPorEnergiaDisponible_USD_MWh );
  f.wr('PagoPorEnergiaEntregada_USD_MWh', PagoPorEnergiaEntregada_USD_MWh);

end;

procedure TParqueEolico_vxy.PrepararMemoria(globs: TGlobs);
var
  jdir: integer;
  iposte: integer;
  auxf: TFR2enR;
begin
  inherited prepararMemoria(globs);
  setlength(PPA, globs.NPostes);

  if not ((fuenteDeViento.durPasoDeSorteoEnHoras = 1) or
    ((fuenteDeViento.durPasoDeSorteoEnHoras = 0) and (globs.HorasDelPaso = 1))) then
    raise Exception.Create(mesFuenteAleatConPaso1h + mesElParque +
      nombre + mesConectadoAFuente + fuenteDeViento.nombre +
      mesConPasoDeSorteo + IntToStr(fuenteDeViento.durPasoDeSorteoEnHoras));

  if kBorneVelocidad_vx = -1 then // la primera vez definimos los bornes
  begin
    kBorneVelocidad_vx := fuenteDeViento.idBorne(nombreBorne_vx);
    kBorneVelocidad_vy := fuenteDeViento.idBorne(nombreBorne_vy);

    setlength(fxPerdidasAerodinamicas, length(fPerdidasAerodinamicas));
    for jdir := 0 to high(fPerdidasAerodinamicas) do
      fxPerdidasAerodinamicas[jdir] := fPerdidasAerodinamicas[jdir] * fMultV;


    {$IFDEF CALC_DEMANDA_NETA}
    auxf := TFf_xmult_ConSelectorYBuffer_vxy.Create(capa, CurvaVP,
      fxPerdidasAerodinamicas, 1, @globs.MesInicioDelPaso,
      @PHoraria_PreSorteosPorUnidad, @globs.kSubPaso_);
    kBornePotencia := fuenteDeViento.registrarFuncion_biborne(
      auxf, nombreBorne_vx, nombreBorne_vy, -1);

    fuenteResumirMaxVar:= fuenteDeViento.ResumirMaxVer( globs );
    if fuenteResumirMaxVar <> nil then
       fuenteResumirMaxVar.flg_ResumirBorneras:= false;

    {$ELSE}
    SetLength(kBornePPA, globs.NPostes);
    for iposte := 0 to high(kBornePPA) do
    begin
       (*
       Para cada poste definimos un borne auxiliar que llevará la cuenta
       de la energía generada por el parque en ese poste y así puede calcular
       la potencia media equivalente del parque para ese poste.
       Se supone que la fuente de viento tiene paso de sorteo HORARIO.
       *)

      auxf := TFf_xmult_conselector_vxy.Create(capa, CurvaVP,
        fxPerdidasAerodinamicas, 1, @globs.MesInicioDelPaso, iposte,
        @globs.kPosteHorasDelPaso, @globs.kSubPaso_);

      kBornePPA[iposte] := fuenteDeViento.registrarFuncion_biborne(
        auxf, nombreBorne_vx, nombreBorne_vy, iposte);
    end;
    {$ENDIF}
  end;
end;

function TParqueEolico_vxy.PotenciaFirme: NReal;
begin
  Result := 0;
end;

function TParqueEolico_vxy.InfoAd_: string;
begin
  Result := '';
end;

class function TParqueEolico_vxy.DescClase: string;
begin
  Result := rsParqueEolico_vxy;
end;


function TParqueEolico_vxy.get_pa_FD( kTipoUnidad: integer ): NReal;
begin
  result:= fdisp;
end;

function TParqueEolico_vxy.get_pa_TMR( kTipoUnidad: integer ): NReal;
begin
  result:= tRepHoras;
end;


procedure TParqueEolico_vxy.SorteosDelPaso(sortear: boolean);
begin
  if globs.ObligarDisponibilidad_1_ then
    NMaquinasDisponibles := paUnidades.nUnidades_Operativas[0]
  else if (sortear) then
  begin
    ActualizarProbabilidadesReparacionYRotura_(fdisp, tRepHoras);
    NMaquinasDisponibles := Sorteos_RepRotUnidades;
  end
  else
    NMaquinasDisponibles := paUnidades.nUnidades_Operativas[0];
  inherited SorteosDelPaso( sortear );
end;



procedure TParqueEolico_vxy.PrepararPaso_ps;
var
  iposte: integer;
  jHora: integer;


begin
  inherited PrepararPaso_ps;

  VVel_x := Self.fuenteDeViento.bornera[Self.kBorneVelocidad_vx];
  VVel_y := Self.fuenteDeViento.bornera[Self.kBorneVelocidad_vy];

  {$IFNDEF CALC_DEMANDA_NETA}
  EnergiaDisponibleDelPaso:= 0;
  for iposte := 0 to high(PPA) do
  begin
    PPA[iposte] := Self.fuenteDeViento.bornera[Self.kBornePPA[iposte]] *
      NMaquinasDisponibles;
    EnergiaDisponibleDelPaso:=  EnergiaDisponibleDelPaso + PPA[iposte] * globs.DurPos[iposte];
  end;
  {$ENDIF}

end;




procedure TParqueEolico_vxy.opt_nvers(var ivar, ivae, ires: integer);
begin
  self.ivar := ivar;
  self.ivae := ivae;
  self.ires := ires;
  ivar := ivar + globs.NPostes;
end;

function TParqueEolico_vxy.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  if (ivar >= self.ivar) and (ivar < self.ivar + globs.NPostes) then
  begin
    nombre := self.Nombre + '_P[MW]' + IntToStr(ivar - self.ivar + 1);
    Result := True;
  end
  else
    Result := False;
end;

class function TParqueEolico_vxy.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list:TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TParqueEolico_vxy, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('fdisp', TParqueEolico_vxy(nil).fdisp, TDataTypeDoubleOfCosa, 113, -1);
  list.AddDataColumn('tRepHoras', TParqueEolico_vxy(nil).tRepHoras, TDataTypeDoubleOfCosa, 113, -1);
  list.AddDataColumn('fPerdidasAerodinamicas', TParqueEolico_vxy(nil).fPerdidasAerodinamicas, TDataTypeDAOfDoubleOfCosa, 113, -1);
  list.AddDataColumn('fMultV', TParqueEolico_vxy(nil).fMultV, TDataTypeDoubleOfCosa, 113, -1);

(* OJO !!! ??? FEDE DANIEL arreglar esto. CurvaP no está inicializada como vector en este momento
  list.AddDataColumn('CurvaVP_N', TParqueEolico_vxy(nil).CurvaVP[0], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_NNE', TParqueEolico_vxy(nil).CurvaVP[1], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_NE', TParqueEolico_vxy(nil).CurvaVP[2], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_ENE', TParqueEolico_vxy(nil).CurvaVP[3], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_E', TParqueEolico_vxy(nil).CurvaVP[4], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_ESE', TParqueEolico_vxy(nil).CurvaVP[5], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_SE', TParqueEolico_vxy(nil).CurvaVP[6], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_SSE', TParqueEolico_vxy(nil).CurvaVP[7], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_S', TParqueEolico_vxy(nil).CurvaVP[8], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_SSO', TParqueEolico_vxy(nil).CurvaVP[9], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_SO', TParqueEolico_vxy(nil).CurvaVP[10], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_OSO', TParqueEolico_vxy(nil).CurvaVP[11], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_O', TParqueEolico_vxy(nil).CurvaVP[12], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_ONO', TParqueEolico_vxy(nil).CurvaVP[13], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_NO', TParqueEolico_vxy(nil).CurvaVP[14], TDataTypeDataRowOfCosa, 113, -1);
  list.AddDataColumn('CurvaVP_NNO', TParqueEolico_vxy(nil).CurvaVP[15], TDataTypeDataRowOfCosa, 113, -1);
 *)

  list.AddDataColumn('fuenteDeViento', TParqueEolico_vxy(nil).fuenteDeViento, TDataTypeReferenciaOfCosa, 113, -1);
  list.AddDataColumn('nombreBorne_vx', TParqueEolico_vxy(nil).nombreBorne_vx, TDataTypeStringOfCosa, 113, -1);
  list.AddDataColumn('nombreBorne_vy', TParqueEolico_vxy(nil).nombreBorne_vy, TDataTypeStringOfCosa, 113, -1);
  list.AddDataColumn('PagoPorEnergiaDisponible_USD_MWh', TParqueEolico_vxy(nil).PagoPorEnergiaDisponible_USD_MWh, TDataTypeDoubleOfCosa, 113, -1);
  list.AddDataColumn('PagoPorEnergiaEntregada_USD_MWh', TParqueEolico_vxy(nil).PagoPorEnergiaEntregada_USD_MWh, TDataTypeDoubleOfCosa, 113, -1);

  result:=list;
end;


procedure TParqueEolico_vxy.opt_cargue(s: TSimplex);
var
  iposte: integer;
begin
  if NMaquinasDisponibles = 0 then
    exit;
{$IFDEF SPXCONLOG}
  spx_NombrarVariables(s);
{$ENDIF}
  for iposte := 0 to globs.NPostes - 1 do
    s.pon_e(nodo.ires + iposte, ivar + iposte, 1);
end;

procedure TParqueEolico_vxy.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  iposte: integer;
begin
  if NMaquinasDisponibles = 0 then
    exit;
  // Le fijamos como cota superior PMax a la potencia en todos los postes
  for iposte := 0 to globs.NPostes - 1 do
    if PPA[iposte] > AsumaCero then
      s.cota_sup_set(ivar + iposte, PPA[iposte])
    else
      s.FijarVariable(ivar + iposte, 0);
end;

procedure TParqueEolico_vxy.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;
  e, eacum: NReal;
begin
  // recuperamos los valores de Potencia despachada
  eacum := 0;
  for iposte := 0 to globs.NPostes - 1 do
  begin
    e := s.xval(ivar + iposte);
    P[iposte] := e;
    eacum := eacum + e * globs.DurPos[iposte];
  end;

  Ingreso_PorDisponibilidad:= EnergiaDisponibleDelPaso * PagoPorEnergiaDisponible_USD_MWh;
  Ingreso_PorEnergia:= eacum * PagoPorEnergiaEntregada_USD_MWh;
  costoDirectoDelPaso := Ingreso_PorDisponibilidad + Ingreso_PorEnergia;
end;

{$IFDEF SPXCONLOG}
procedure TParqueEolico_vxy.spx_NombrarVariables(s: TSimplex);
var
  iposte: integer;
begin
  for iposte := 0 to globs.NPostes - 1 do
    s.set_NombreVar(ivar + iposte, Nombre + '_P[MW]' + IntToStr(iposte + 1));
end;

{$ENDIF}

procedure TParqueEolico_vxy.dump_Variables(var f: TextFile; charIndentacion: char);
var
  i: integer;
begin
  inherited dump_Variables(f, charIndentacion);
  writeln(f, charIndentacion, 'VVel_vx[m/s]= ', FloatToStrF(VVel_x, ffFixed, 10, 3));
  writeln(f, charIndentacion, 'VVel_vy[m/s]= ', FloatToStrF(VVel_y, ffFixed, 10, 3));
  for i := 0 to high(PPA) do
    writeln(f, charIndentacion, 'PPA[', i, '][MW]: ',
      FloatToStrF(PPA[i], ffFixed, 10, 3));
  writeln(f);
  //  writeln(f, charIndentacion, 'PotMediaGenerable[MW]= ', FloatToStrF(PotMediaGenerable, ffFixed, 10, 3));
  writeln(f, charIndentacion, 'NMaquinasDisponibles= ', NMaquinasDisponibles);
  writeln(f);
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TParqueEolico_vxy.ClassName, TParqueEolico_vxy);
end;

procedure AlFinal;
begin
end;

end.
