unit ubiomasaembalsable;

{$mode delphi}


interface

uses
  Math,
  SysUtils, Classes, xmatdefs, uGTer, uNodos,
  uglobs,
  usimplex,
  ufichasLPD,
  ufechas,
  ucosa, uranddispos,
  uconstantesSimSEE,
  uFuentesAleatorias, uActores, uEstados;

resourcestring
  rsGeneradorBiomasaEmbalsable =
    'Generador de Biomasa Embalsable';


type

  { TFichaBiomasaEmbalsable }

  TFichaBiomasaEmbalsable = class(TFichaLPD)
  public
    PMax: NReal;// MW máximos de una unidad generadora
    PEC: NReal; // [USD/MWh] Pago por Energía Comprometida
    PEE: NReal; // [USD/MWh] Pago por Energía Entregada (o vertida TOP)
    PEX: NREal; // [USD/MWh] Premio por Externalidades.
    fd: NReal;  // factor de disponibilidad fortuita
    TMR: NReal; // Tiempo Medio de Reparción (horas)
    cvea_impuesto: NReal; // [USD/MWh] valor de la energía almacenada
    // si flg_imponer_cvea = true.

    a_PEC, b_PEC, c_PEC, d_PEC: NReal;  // poderadores indexación PEC
    a_PEE, b_PEE, c_PEE, d_PEE: NReal;  // poderadores indexación PEE
    a_PEX, b_PEX, c_PEX, d_PEX: NReal;  // poderadores indexación PEX

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      fd, TMR: NReal; PMax: NReal; PEC, PEE, PEX, cvea_impuesto: NReal;
      a_PEC, b_PEC, c_PEC, d_PEC: NReal;  // poderadores indexación PEC
      a_PEE, b_PEE, c_PEE, d_PEE: NReal;  // poderadores indexación PEE
      a_PEX, b_PEX, c_PEX, d_PEX: NReal  // poderadores indexación PEX
      );
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure generarLineaResumen(var archi: TextFile); override;
    function infoAd_: string; override;
    procedure Free; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;

  end;

  { TBiomasaEmbalsable }

  TBiomasaEmbalsable = class(TGTer)
  public

    // parametros persistentes
    X_EA_inicial: NReal;  // Valor del estado al inicio.
    ECA_DOPTOP: NReal; // Energía comprometida ANUAL
    meses_TOP: integer; // meses máximo que se mantiene el compromiso por energía.
    NPuntos_EA: integer; // Cantidad de puntos de discretización de X_EA

    ixA_fuente: TFuenteAleatoria;
    ixA_Borne_nombre: string;
    ixA_Borne_numero: integer;

    ixB_fuente: TFuenteAleatoria;
    ixB_Borne_nombre: string;
    ixB_Borne_numero: integer;

    ixC_fuente: TFuenteAleatoria;
    ixC_Borne_nombre: string;
    ixC_Borne_numero: integer;

    ixD_fuente: TFuenteAleatoria;
    ixD_Borne_nombre: string;
    ixD_Borne_numero: integer;

    flg_imponer_cvea: boolean;

    pa: TFichaBiomasaEmbalsable;

    PEC, PEE, PEX: NReal; // USD/MWh indexados.
    cvea_impuesto: NReal; // USD/MWh indexado.

    ecp: NReal; // energía comprometida del paso.
    edp: NReal; // energía despachable del paso. PN * HorasDelPaso
    EA_Max: NReal; // Volumen máximo de energía almacenada;

    // Variable de estado  Energia Almaceanda MWh
    X_EA: NReal;
    Xs_EA: NReal;

    PNominalPorUnidad: NReal; // es pa.PMax salvo que sea sin sorteos
    NMaquinasDisponibles: integer; // maximo de acople
    NMaquinasDespachables: integer; // Para cuantas da la energía del paso

    // -dCF/dX
    cvea: NReal;

    // REsultados del paso
    NMaquinasDespachadas: integer;
    EnergiaVertida_TOP,
    EnergiaDespachada_DOP: NReal;
    Pagos_TOP: NReal;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean;
  ECA_DOPTOP: NReal;
  meses_TOP: integer; NPuntosEA: integer; X_EA_Inicial: NReal;
      ixA_fuente: TFuenteAleatoria; ixA_Borne_nombre: string;
      ixB_fuente: TFuenteAleatoria; ixB_Borne_nombre: string;
      ixC_fuente: TFuenteAleatoria; ixC_Borne_nombre: string;
      ixD_fuente: TFuenteAleatoria; ixD_Borne_nombre: string;
  imponer_cvea: boolean; TonCO2xMWh: NReal; LowCostMustRun,
  CleanDevelopmentMechanism: boolean); reintroduce;

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure RegistrarParametrosDinamicos; override;

    function PotenciaFirme: NReal; override;

    function InfoAd_: string; override;
    class function DescClase: string; override;

    procedure PosicionarseEnEstrellita; override;
    procedure ActualizarEstadoGlobal( flg_Xs: boolean ); override;

    procedure Sim_Cronica_Inicio; override;
    procedure SorteosDelPaso(sortear: boolean); override;
    procedure PrepararPaso_ps; override;

    procedure EvolucionarEstado; override;

    //      procedure Sim_Cronica_Fin; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;
    procedure optx_nvxs(var ixr, ixd, iauxNReal, iauxInt: integer); override;
    procedure optx_RegistrarVariablesDeEstado(adminEstados: TAdminEstados);
      override;

    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;
{$IFDEF SPXCONLOG}
    procedure spx_NombrarVariables(s: TSimplex); override;
{$ENDIF}
    function getNombreVar(ivar: integer; var nombre: string): boolean; override;
    function getNombreRes(ires: integer; var nombre: string): boolean; override;

    procedure Free; override;

    procedure PubliVars; override;
    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    class function TipoFichaLPD: TClaseDeFichaLPD; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
  end;

procedure cambioFichaPD_TBiomasaEmbalsable(actor: TActor);
procedure cambioFichaUnidades_TBiomasaEmbalsable(actor: TActor);

procedure AlInicio;
procedure AlFinal;

implementation


//----------------------------------
// Métodos de TFichaBiomasaEmbalsable
//==================================
constructor TFichaBiomasaEmbalsable.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; fd, TMR: NReal; PMax: NReal;
  PEC, PEE, PEX, cvea_impuesto: NReal;
  a_PEC, b_PEC, c_PEC, d_PEC: NReal;  // poderadores indexación PEC
  a_PEE, b_PEE, c_PEE, d_PEE: NReal;  // poderadores indexación PEE
  a_PEX, b_PEX, c_PEX, d_PEX: NReal  // poderadores indexación PEX
  );
begin
  inherited Create(capa, fecha, periodicidad);
  self.fd := fd;
  self.TMR := TMR;
  self.PMax := PMax;
  self.PEC := PEC;
  self.PEE := PEE;
  self.PEX := PEX;
  self.cvea_impuesto := cvea_impuesto;
  self.a_PEC:= a_PEC;
  self.b_PEC:= b_PEC;
  self.c_PEC:= c_PEC;
  self.d_PEC:= d_PEC;

  self.a_PEE:= a_PEE;
  self.b_PEE:= b_PEE;
  self.c_PEE:= c_PEE;
  self.d_PEE:= d_PEE;

  self.a_PEX:= a_PEX;
  self.b_PEX:= b_PEX;
  self.c_PEX:= c_PEX;
  self.d_PEX:= d_PEX;

end;

constructor TFichaBiomasaEmbalsable.Create_ReadFromText(f: TArchiTexto);
begin

  inherited Create_ReadFromText(f);

  f.IniciarLecturaRetrasada;
  f.rd('fd', fd);
  f.rd('TMR', TMR);
  f.rd('PMax', PMax);
  f.rd('PEC', PEC);
  f.rd('PEE', PEE);
  if f.version >= 123 then
    f.rd('PEX', PEX)
  else
    PEX:= 0;


  if f.version > 106 then
    f.rd('cvea_impuesto', cvea_impuesto)
  else
    cvea_impuesto := 0;

  if f.version >= 123 then
  begin
    f.rd( 'a_PEC', a_PEC );
    f.rd( 'b_PEC', b_PEC );
    f.rd( 'c_PEC', c_PEC );
    f.rd( 'd_PEC', d_PEC );

    f.rd( 'a_PEE', a_PEE );
    f.rd( 'b_PEE', b_PEE );
    f.rd( 'c_PEE', c_PEE );
    f.rd( 'd_PEE', d_PEE );

    f.rd( 'a_PEX', a_PEX );
    f.rd( 'b_PEX', b_PEX );
    f.rd( 'c_PEX', c_PEX );
    f.rd( 'd_PEX', d_PEX );

  end
  else
  begin
    self.a_PEC:= 1;
    self.b_PEC:= 0;
    self.c_PEC:= 0;
    self.d_PEC:= 0;

    self.a_PEE:= 0;
    self.b_PEE:= 1;
    self.c_PEE:= 0;
    self.d_PEE:= 0;

    self.a_PEX:= 0;
    self.b_PEX:= 1;
    self.c_PEX:= 0;
    self.d_PEX:= 0;
  end;

  f.EjecutarLectura;
end;

procedure TFichaBiomasaEmbalsable.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('fd', fd);
  f.wr('TMR', TMR);
  f.wr('PMax', PMax);
  f.wr('PEC', PEC);
  f.wr('PEE', PEE);
  f.wr('PEX', PEX);
  f.wr('cvea_impuesto', cvea_impuesto);

  f.wr( 'a_PEC', a_PEC );
  f.wr( 'b_PEC', b_PEC );
  f.wr( 'c_PEC', c_PEC );
  f.wr( 'd_PEC', d_PEC );

  f.wr( 'a_PEE', a_PEE );
  f.wr( 'b_PEE', b_PEE );
  f.wr( 'c_PEE', c_PEE );
  f.wr( 'd_PEE', d_PEE );

  f.wr( 'a_PEX', a_PEX );
  f.wr( 'b_PEX', b_PEX );
  f.wr( 'c_PEX', c_PEX );
  f.wr( 'd_PEX', d_PEX );

end;

procedure TFichaBiomasaEmbalsable.generarLineaResumen(var archi: TextFile);
begin
  Write(archi, '-', #9,       //PMín
    FloatToStrF(PMax, formatoReales, 8, 1), #9,  //PMáx
    '-', #9,       //CV_Mín
    FloatToStrF(PEC + PEE, formatoReales, 8, 2), #9,    //CV_Medio
    FloatToStrF(PEE, formatoReales, 8, 2), #9,    //CV_Incremental
    FloatToStrF(fd, formatoReales, 8, 2), #9,  //FDisp
    '-', #9,       //Costo Arranque
    '-', #9,       //Costo Parada
    '-', #9,       //mínNPasosOn
    '-', #9,       //mínNPasosOff
    '-', #9,       //desiciónPasosOnPorCiclo
    '-', #9,       //desiciónPasosOffPorCiclo
    '-', #9,       //costoPorCicloOn
    '-', #9);      //costoPorCicloOff
end;

function TFichaBiomasaEmbalsable.infoAd_: string;
begin
  Result := 'PMáx= ' + FloatToStrF(PMax, ffGeneral, 10, 1) + ' MW, ' +
    'PE= ' + FloatToStrF(PEC + PEE, ffGeneral, 10, 1) + ' USD/MWh, ' +
    'fd= ' + FloatToStrF(fd, ffGeneral, 10, 2) + ' p.u., ' + 'TMR= ' +
    FloatToStrF(TMR, ffGeneral, 10, 1) + ' hs';
end;

procedure TFichaBiomasaEmbalsable.Free;
begin
  inherited Free;
end;

class function TFichaBiomasaEmbalsable.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TFichaBiomasaEmbalsable, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('fd', TFichaBiomasaEmbalsable(nil).fd, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('TMR', TFichaBiomasaEmbalsable(nil).TMR, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('PMax', TFichaBiomasaEmbalsable(nil).PMax, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('PEC', TFichaBiomasaEmbalsable(nil).PEC, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('PEE', TFichaBiomasaEmbalsable(nil).PEE, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('PEX', TFichaBiomasaEmbalsable(nil).PEX, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('cvea_impuesto', TFichaBiomasaEmbalsable(nil).cvea_impuesto, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('a_PEC', TFichaBiomasaEmbalsable(nil).a_PEC, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('b_PEC', TFichaBiomasaEmbalsable(nil).b_PEC, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('c_PEC', TFichaBiomasaEmbalsable(nil).c_PEC, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('d_PEC', TFichaBiomasaEmbalsable(nil).d_PEC, TDataTypeDoubleOfCosa, 1, -1);

  list.AddDataColumn('a_PEE', TFichaBiomasaEmbalsable(nil).a_PEE, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('b_PEE', TFichaBiomasaEmbalsable(nil).b_PEE, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('c_PEE', TFichaBiomasaEmbalsable(nil).c_PEE, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('d_PEE', TFichaBiomasaEmbalsable(nil).d_PEE, TDataTypeDoubleOfCosa, 1, -1);

  list.AddDataColumn('a_PEX', TFichaBiomasaEmbalsable(nil).a_PEX, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('b_PEX', TFichaBiomasaEmbalsable(nil).b_PEX, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('c_PEX', TFichaBiomasaEmbalsable(nil).c_PEX, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('d_PEX', TFichaBiomasaEmbalsable(nil).d_PEX, TDataTypeDoubleOfCosa, 1, -1);

  Result:=list;
end;

//-----------------------------
// Métodos de TBiomasaEmbalsable
//=============================
procedure TBiomasaEmbalsable.dump_Variables(var f: TextFile; charIndentacion: char);
begin
  inherited dump_Variables(f, charIndentacion);
  writeln(f, charIndentacion, 'PEC[USD/MWh]= ', FloatToStrF(PEC, ffFixed, 10, 3));
  writeln(f, charIndentacion, 'PMAx[MW]= ', FloatToStrF(pa.PMax, ffFixed, 10, 3));
  writeln(f, charIndentacion, 'X_EA= ', X_EA);
  writeln(f);
end;

class function TBiomasaEmbalsable.TipoFichaLPD: TClaseDeFichaLPD;
begin
  Result := TFichaBiomasaEmbalsable;
end;

class function TBiomasaEmbalsable.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TBiomasaEmbalsable, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('ECA_DOPTOP', TBiomasaEmbalsable(nil).ECA_DOPTOP, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('meses_TOP', TBiomasaEmbalsable(nil).meses_TOP, TDataTypeIntegerOfCosa, 1, -1);
  list.AddDataColumn('NPuntos_EA', TBiomasaEmbalsable(nil).NPuntos_EA, TDataTypeIntegerOfCosa, 1, -1);
  list.AddDataColumn('X_EA_inicial', TBiomasaEmbalsable(nil).X_EA_inicial, TDataTypeDoubleOfCosa, 1, -1);

  list.AddDataColumn('ixA_fuente', TBiomasaEmbalsable(nil).ixA_fuente, TDataTypeReferenciaOfCosa, 1, -1);
  list.AddDataColumn('ixA_Borne_nombre', TBiomasaEmbalsable(nil).ixA_Borne_nombre, TDataTypeStringOfCosa, 1, -1);

  list.AddDataColumn('ixB_fuente', TBiomasaEmbalsable(nil).ixB_fuente, TDataTypeReferenciaOfCosa, 1, -1);
  list.AddDataColumn('ixB_Borne_nombre', TBiomasaEmbalsable(nil).ixB_Borne_nombre, TDataTypeStringOfCosa, 1, -1);

  list.AddDataColumn('ixC_fuente', TBiomasaEmbalsable(nil).ixC_fuente, TDataTypeReferenciaOfCosa, 1, -1);
  list.AddDataColumn('ixC_Borne_nombre', TBiomasaEmbalsable(nil).ixC_Borne_nombre, TDataTypeStringOfCosa, 1, -1);

  list.AddDataColumn('ixD_fuente', TBiomasaEmbalsable(nil).ixD_fuente, TDataTypeReferenciaOfCosa, 1, -1);
  list.AddDataColumn('ixD_Borne_nombre', TBiomasaEmbalsable(nil).ixD_Borne_nombre, TDataTypeStringOfCosa, 1, -1);

  list.AddDataColumn('flg_imponer_cvea', TBiomasaEmbalsable(nil).flg_imponer_cvea, TDataTypeBooleanOfCosa, 1, -1);

  Result:=list;


end;

constructor TBiomasaEmbalsable.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean;
  ECA_DOPTOP: NReal;
  meses_TOP: integer;
  NPuntosEA: integer; X_EA_Inicial: NReal;
  ixA_fuente: TFuenteAleatoria; ixA_Borne_nombre: string;
  ixB_fuente: TFuenteAleatoria; ixB_Borne_nombre: string;
  ixC_fuente: TFuenteAleatoria; ixC_Borne_nombre: string;
  ixD_fuente: TFuenteAleatoria; ixD_Borne_nombre: string;

  imponer_cvea: boolean; TonCO2xMWh: NReal;
  LowCostMustRun, CleanDevelopmentMechanism: boolean);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, lpd, nodo,
    flg_CalcularGradienteDeInversion,
    TonCO2xMWh, LowCostMustRun, CleanDevelopmentMechanism);
  P := nil;
  self.ECA_DOPTOP := ECA_DOPTOP;
  self.meses_TOP:= meses_TOP;
  self.NPuntos_EA := NPuntos_EA;
  self.X_EA_Inicial := X_EA_Inicial;

  self.ixA_fuente := ixA_fuente;
  self.ixA_Borne_nombre := ixA_Borne_nombre;
  self.ixA_Borne_numero := -1;

  self.ixB_fuente := ixB_fuente;
  self.ixB_Borne_nombre := ixB_Borne_nombre;
  self.ixB_Borne_numero := -1;

  self.ixC_fuente := ixC_fuente;
  self.ixC_Borne_nombre := ixC_Borne_nombre;
  self.ixC_Borne_numero := -1;

  self.ixD_fuente := ixD_fuente;
  self.ixD_Borne_nombre := ixD_Borne_nombre;
  self.ixD_Borne_numero := -1;

  self.flg_imponer_cvea := imponer_cvea;
end;

constructor TBiomasaEmbalsable.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);


  if f.version >= 123 then
  begin
    f.rd('ECA_DOPTOP', ECA_DOPTOP);
  end
  else
  begin
    f.rd('ECA', ECA_DOPTOP);
  end;


  if f.Version > 108 then
  begin
     f.rd('meses_TOP', meses_TOP );
     f.rd('NPuntos_EA', NPuntos_EA);
  end
  else
  begin
     meses_TOP:= 12;
     f.rd('NPuntos_ECA', NPuntos_EA);
  end;



  f.rd('X_EA_inicial', X_EA_inicial);

  if f.version >= 123 then
  begin
    f.rdReferencia('ixA_fuente', TCosa(ixA_fuente), self);
    f.rd('ixA_Borne_nombre', ixA_Borne_nombre);
    f.rdReferencia('ixB_fuente', TCosa(ixB_fuente), self);
    f.rd('ixB_Borne_nombre', ixB_Borne_nombre);
    f.rdReferencia('ixC_fuente', TCosa(ixC_fuente), self);
    f.rd('ixC_Borne_nombre', ixC_Borne_nombre);
    f.rdReferencia('ixD_fuente', TCosa(ixD_fuente), self);
    f.rd('ixD_Borne_nombre', ixD_Borne_nombre);
  end
  else
  begin
    f.rdReferencia('ixPEC_fuente', TCosa(ixA_fuente), self);
    f.rd('ixPEC_Borne_nombre', ixA_Borne_nombre);
    f.rdReferencia('ixPEE_fuente', TCosa(ixB_fuente), self);
    f.rd('ixPEE_Borne_nombre', ixB_Borne_nombre);
    ixC_fuente:= nil; ixC_Borne_nombre:= ''; ixC_Borne_numero:= -1;
    ixD_fuente:= nil; ixD_Borne_nombre:= ''; ixD_Borne_numero:= -1;
  end;

  if f.Version > 106 then
    f.rd('flg_imponer_cvea', flg_imponer_cvea)
  else
    flg_imponer_cvea := False;

  P := nil;
  pa := nil;
  nodo := nil;
end;

procedure TBiomasaEmbalsable.PrepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
  if ixA_fuente <> nil then
    ixA_Borne_numero := ixA_fuente.IdBorne(ixA_Borne_nombre);
  if ixB_fuente <> nil then
    ixB_Borne_numero := ixB_fuente.IdBorne(ixB_Borne_nombre);
  if ixC_fuente <> nil then
    ixC_Borne_numero := ixC_fuente.IdBorne(ixC_Borne_nombre);
  if ixD_fuente <> nil then
    ixD_Borne_numero := ixD_fuente.IdBorne(ixD_Borne_nombre);
end;

procedure TBiomasaEmbalsable.RegistrarParametrosDinamicos;
begin
  uActores.procCambioFichaUnidades := @cambioFichaUnidades_TBiomasaEmbalsable;
  inherited registrarParametrosDinamicos;
  uActores.procCambioFichaUnidades := nil;
  lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA,
    nil, @cambioFichaPD_TBiomasaEmbalsable);
end;

function TBiomasaEmbalsable.PotenciaFirme: NReal;
begin
  Result := (paUnidades.nUnidades_Operativas[0]) * pa.PMax * pa.fd;
end;

function TBiomasaEmbalsable.InfoAd_: string;
begin
  Result := '';
end;

class function TBiomasaEmbalsable.DescClase: string;
begin
  Result := rsGeneradorBiomasaEmbalsable;
end;

procedure TBiomasaEmbalsable.PosicionarseEnEstrellita;
begin
  if not flg_imponer_cvea then
    X_EA := globs.CF.xr[ixr]
  else
    X_EA := ECA_DOPTOP / 2.0;
end;

procedure TBiomasaEmbalsable.ActualizarEstadoGlobal( flg_Xs: boolean );
begin
  if not flg_imponer_cvea then
    globs.CF.xr[ixr] := X_EA;
end;

procedure TBiomasaEmbalsable.Sim_Cronica_Inicio;
begin
  inherited Sim_Cronica_Inicio;
  X_EA := X_EA_inicial;
end;

procedure TBiomasaEmbalsable.SorteosDelPaso(sortear: boolean);
begin
  if hayForzamientos or globs.ObligarDisponibilidad_1_ then
  begin
    NMaquinasDisponibles := paUnidades.nUnidades_Operativas[0];
    PNominalPorUnidad := pa.PMax;
  end
  else if sortear then
  begin
    ActualizarProbabilidadesReparacionYRotura_(pa.fd, pa.TMR);
    NMaquinasDisponibles := Sorteos_RepRotUnidades;
    PNominalPorUnidad := pa.PMax;
  end
  else
  begin
    NMaquinasDisponibles := paUnidades.nUnidades_Operativas[0];
    PNominalPorUnidad := pa.PMax * pa.fd;
  end;
end;

procedure TBiomasaEmbalsable.PrepararPaso_ps;
var
  dCFdX_inc, dCFdX_dec: NReal;
  rescod: integer;
  xrpos: NReal;
  dCFdX: NReal;
  actualizador: NReal;
  residuo: NReal;

procedure addcomp( fuente: TFuenteAleatoria; borne: integer; factor: NReal );
begin
  if fuente <> nil then
  begin
    actualizador:= actualizador + factor * fuente.Bornera[borne];
    residuo:= residuo - factor;
  end;
end;

begin

  // calculamos los valores PEC y PEE en base los precios de la ficha
  // de parámetros dinámicos activa y los valores de las fuentes de
  // indexación de ambos precios.
  residuo:= 1;
  actualizador:= 0;
  addcomp(  ixA_fuente, ixA_Borne_numero, pa.a_PEC );
  addcomp(  ixB_fuente, ixB_Borne_numero, pa.b_PEC );
  addcomp(  ixC_fuente, ixC_Borne_numero, pa.c_PEC );
  addcomp(  ixD_fuente, ixD_Borne_numero, pa.d_PEC );
  actualizador:= actualizador + residuo;
  PEC:= pa.PEC * actualizador;

  residuo:= 1;
  actualizador:= 0;
  addcomp(  ixA_fuente, ixA_Borne_numero, pa.a_PEE );
  addcomp(  ixB_fuente, ixB_Borne_numero, pa.b_PEE );
  addcomp(  ixC_fuente, ixC_Borne_numero, pa.c_PEE );
  addcomp(  ixD_fuente, ixD_Borne_numero, pa.d_PEE );
  actualizador:= actualizador + residuo;
  PEE:= pa.PEE * actualizador;
  if flg_imponer_cvea then
    cvea_impuesto := pa.cvea_impuesto * actualizador;


  residuo:= 1;
  actualizador:= 0;
  addcomp(  ixA_fuente, ixA_Borne_numero, pa.a_PEX );
  addcomp(  ixB_fuente, ixB_Borne_numero, pa.b_PEX );
  addcomp(  ixC_fuente, ixC_Borne_numero, pa.c_PEX );
  addcomp(  ixD_fuente, ixD_Borne_numero, pa.d_PEX );
  actualizador:= actualizador + residuo;
  PEX:= pa.PEX * actualizador;


  if not flg_imponer_cvea then
  begin
  {$IFDEF DEV_CF_ESTRELLA}
    if globs.EstadoDeLaSala = CES_OPTIMIZANDO then
      globs.CF.devxr_estrella_(ixr, globs.kPaso_ + 1, dCFdX_inc, dCFdX_dec, rescod)
    else
  {$ENDIF}
      globs.CF.devxr_continuo(ixr, globs.kPaso_ + 1, dCFdX_inc,
        dCFdX_dec, rescod, xrpos);

    case rescod of
      -1: dCFdX := dCFdX_inc;
      0: dCFdX := (dCFdX_dec + dCFdX_inc) / 2.0;
      1: dCFdX := dCFdX_dec;
    end;

    cvea := -dCFdX * globs.fActPaso;
  end
  else
    cvea := cvea_impuesto;

  if NMaquinasDisponibles = 0 then
    NMaquinasDespachables := 0
  else
  begin
    NMaquinasDespachables := trunc((X_EA + ecp) / edp);
    NMaquinasDespachables := min(NMaquinasDisponibles, NMaquinasDespachables);
  end;

end;

procedure TBiomasaEmbalsable.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('ECA_DOPTOP', ECA_DOPTOP);
  f.wr('meses_TOP', meses_TOP );
  f.wr('NPuntos_EA', NPuntos_EA);
  f.wr('X_EA_inicial', X_EA_inicial);

  f.wrReferencia('ixA_fuente', TCosa(ixA_fuente));
  f.wr('ixA_Borne_nombre', ixA_Borne_nombre);
  f.wrReferencia('ixB_fuente', TCosa(ixB_fuente));
  f.wr('ixB_Borne_nombre', ixB_Borne_nombre);
  f.wrReferencia('ixC_fuente', TCosa(ixC_fuente));
  f.wr('ixC_Borne_nombre', ixC_Borne_nombre);
  f.wrReferencia('ixD_fuente', TCosa(ixD_fuente));
  f.wr('ixD_Borne_nombre', ixD_Borne_nombre);

  f.wr('flg_imponer_cvea', flg_imponer_cvea);
end;

procedure TBiomasaEmbalsable.opt_nvers(var ivar, ivae, ires: integer);
begin
  if NMaquinasDespachables = 0 then
    exit; // si no hay máquinas no juego

  Self.ivar := ivar;
  ivar := ivar + 3; // A, VE, Xs_EA

  self.ivae := ivae;
  Inc(ivae);

  Self.ires := ires;
  Inc(ires);  //   - A* edp -VE -Xs_EA + x_EA + ecp = 0
end;

procedure TBiomasaEmbalsable.optx_nvxs(var ixr, ixd, iauxNReal, iauxInt: integer);
begin
  if not flg_imponer_cvea then
  begin
    Self.ixr := ixr;
    ixr := ixr + 1;
  end;
end;

procedure TBiomasaEmbalsable.optx_RegistrarVariablesDeEstado(
  adminEstados: TAdminEstados);
begin
  if not flg_imponer_cvea then
    adminEstados.Registrar_Continua(ixr, 0, EA_Max , NPuntos_EA, 'EA', 'MWh');
end;

procedure TBiomasaEmbalsable.opt_cargue(s: TSimplex);
var
  inodores: integer;
  iposte: integer;
begin
  if NMaquinasDespachables = 0 then
    exit; // si no hay máquinas no juego

{$IFDEF SPXCONLOG}
  spx_NombrarVariables(s);
{$ENDIF}

  inodores := nodo.ires;
  for iposte := 0 to globs.NPostes - 1 do
  begin
    // cargamos la potencia en la restricción de balance de potencia del nodo
    s.pon_e(inodores + iposte, ivar, PNominalPorUnidad);
  end;

  // Cargamos la restricción dinámica
  // - edp*A -VE -Xs_EA + x_EA + ecp = 0
  s.pon_e(ires, ivar, -edp);
  s.pon_e(ires, ivar + 1, -1);
  s.pon_e(ires, ivar + 2, -1);
  s.pon_e(ires, s.nc, x_EA + ecp);

  // Ahora agregamos a la función de UTILIDAD (-costo
  s.pon_e(s.nf, ivar, -edp * PEE ); // - A * edp  * PEE
  s.pon_e(s.nf, ivar + 1, -PEE ); // - VE * PEE
  s.pon_e(s.nf, ivar + 2, cvea );  // - cvea * (X - Xs )

  // s.acum_e(s.nf, s.nc, -cvea * X_EA - PEC * ecp);

end;

procedure TBiomasaEmbalsable.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  iposte: integer;
  AForzadas: integer;

begin
  if NMaquinasDespachables = 0 then
    exit; // si no hay máquinas no juego


  if (not hayForzamientos) then
  begin
    s.cota_sup_set(ivar, NMaquinasDespachables);
  end
  else
  begin
    AForzadas := trunc(paForzamiento.P[0] / pa.PMax + 0.5);
    s.FijarVariable(ivar, AForzadas);
  end;

  s.set_entera(ivae, ivar, NMaquinasDespachables);

  s.cota_sup_set(ivar + 1, max(0, X_EA + ecp - EA_Max ));
  // No puede verter más que lo que le sobra si no despacha nada

  s.cota_sup_set(ivar + 2, EA_Max); // el estado no puede superar ECA

  s.FijarRestriccionIgualdad(ires);
end;

procedure TBiomasaEmbalsable.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;
  m: NReal;

begin
  Ingreso_PorEnergia := 0;
  Ingreso_PorDisponibilidad := ecp * PEC;
  Pagos_TOP := Ingreso_PorDisponibilidad;


  CostoDirectoDelPaso := Ingreso_PorDisponibilidad;
  if NMaquinasDespachables = 0 then
  begin
    vclear(P);
    NMaquinasDespachadas := 0;
    EnergiaDespachada_DOP := 0;
    Xs_EA := X_EA + ecp;
    if (Xs_EA > EA_Max) then
    begin
      EnergiaVertida_TOP := ( Xs_EA - EA_Max );
      Xs_EA := EA_Max;
      Ingreso_PorEnergia := EnergiaVertida_TOP * PEE;
      Pagos_TOP:= Pagos_TOP + Ingreso_PorEnergia;
      costoDirectoDelPaso := costoDirectoDelPaso + Ingreso_PorEnergia;
    end
    else
      EnergiaVertida_TOP := 0;
    exit;
  end;

  NMaquinasDespachadas := trunc(s.xval(ivar) + 0.1);
  EnergiaVertida_TOP := s.xval(ivar + 1);
  Xs_EA := s.xval(ivar + 2);


  P[0] := NMaquinasDespachadas * PNominalPorUnidad;
  EnergiaDespachada_DOP := NMaquinasDespachadas * edp;


  // recuperamos los valores de Potencia despachada
  for iposte := 1 to globs.NPostes - 1 do
    P[iposte] := P[0];

  Ingreso_PorEnergia := ( EnergiaDespachada_DOP + EnergiaVertida_TOP ) * PEE;
  Pagos_TOP:= Pagos_TOP + ( EnergiaDespachada_DOP + EnergiaVertida_TOP ) * PEE;
  costoDirectoDelPaso := costoDirectoDelPaso + Ingreso_PorEnergia;
end;

procedure TBiomasaEmbalsable.EvolucionarEstado;
begin
  X_EA := Xs_EA;
end;

{$IFDEF SPXCONLOG}
procedure TBiomasaEmbalsable.spx_NombrarVariables(s: TSimplex);
var
  iposte: integer;
begin
  if NMaquinasDespachables = 0 then
    exit;
  s.set_NombreVar(ivar, Nombre + '_Acopladas');
  s.set_NombreVar(ivar + 1, Nombre + '_EnergiaVertida');
  s.set_NombreVar(ivar + 2, Nombre + '_Xs_EA');
  s.set_NombreRest(ires, nombre + 'dinámica(EA)');
end;

{$ENDIF}

function TBiomasaEmbalsable.getNombreVar(ivar: integer; var nombre: string): boolean;
var
  kvar: integer;
  res: boolean;
begin
  res := True;
  kvar := ivar - self.ivar;
  case kvar of
    0: nombre := self.Nombre + '_Acopladas';
    1: nombre := self.Nombre + '_EnergiaVertida';
    2: nombre := self.Nombre + '_Xs_EA';
    else
      Res := False;
  end;
  Result := res;
end;

function TBiomasaEmbalsable.getNombreRes(ires: integer; var nombre: string): boolean;
begin
  if ires = self.ires then
  begin
    nombre := 'dinamica(EA)';
    Result := True;
  end
  else
    Result := False;

end;

procedure TBiomasaEmbalsable.PubliVars;
begin
  inherited PubliVars;

  PublicarVariableNR('X_EA', 'MW', 6, 1, X_EA, True);
  PublicarVariableNI('NMaquinasDespachadas', 'u', NMaquinasDespachadas, True);
  PublicarVariableNR('EnergiaVertida_TOP', 'MWh', 6, 1, EnergiaVertida_TOP, True);
  PublicarVariableNR('EnergiaDespachada_DOP', 'MWh', 6, 1, EnergiaDespachada_DOP, True);
  PublicarVariableNR('cvea', 'USD/MWh', 6, 3, cvea, True);
  PublicarVariableNR('IngresoPEC', 'USD', 6, 1, Ingreso_PorDisponibilidad, True);
  PublicarVariableNR('IngresoPEE', 'USD', 6, 1, Ingreso_PorEnergia, True);
  PublicarVariableNR('Pagos_TOP', 'USD', 6, 1, Pagos_TOP, True);

end;

procedure TBiomasaEmbalsable.Free;
begin
  inherited Free;
end;

procedure cambioFichaPD_TBiomasaEmbalsable(actor: TActor);
begin
  with actor as TBiomasaEmbalsable do
  begin
    ecp := ECA_DOPTOP / HorasDelAnio * globs.HorasDelPaso;
    edp := pa.PMax * globs.HorasDelPaso;
    EA_Max:= ECA_DOPTOP/12 * meses_TOP;
  end;
end;

procedure cambioFichaUnidades_TBiomasaEmbalsable(actor: TActor);
begin
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TBiomasaEmbalsable.ClassName, TBiomasaEmbalsable);
  registrarClaseDeCosa(TFichaBiomasaEmbalsable.ClassName, TFichaBiomasaEmbalsable);
end;

procedure AlFinal;
begin
end;

end.
